﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Esmf
{
    [Serializable]
    public class Parameter2Dimensional<T> : Parameter
    {
        private ParameterElement<T>[,] _value;

        public Parameter2Dimensional(string name, int id, ParameterElement<T>[,] value)
            : base(name, id)
        {
            _value = value;
        }


        public JaggedArrayWrapper2<T> GetBestGuessValues()
        {
            int l0 = _value.GetLength(0);
            int l1 = _value.GetLength(1);

            JaggedArrayWrapper2<T> values = new JaggedArrayWrapper2<T>(l0, l1);

            for (int i = 0; i < l0; i++)
            {
                for (int l = 0; l < l1; l++)
                {
                    values[i,l] = _value[i,l].GetBestGuessValue();
                }
            }

            return values;
        }

        public JaggedArrayWrapper2<T> GetRandomValues(Random rand)
        {
            int l0 = _value.GetLength(0);
            int l1 = _value.GetLength(1);

            JaggedArrayWrapper2<T> values = new JaggedArrayWrapper2<T>(l0, l1);

            for (int i = 0; i < l0; i++)
            {
                for (int l = 0; l < l1; l++)
                {
                    values[i, l] = _value[i, l].GetRandomValue(rand);
                }
            }

            return values;
        }
    }
}