// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Text;

namespace Esmf
{
    public struct Parameter1DimensionalMember<D1,T>
    {
        private IEnumerable<string> _attachedDimensions;
        D1 _dimension1;
        T _value;

        public Parameter1DimensionalMember(D1 dimension, T value, IEnumerable<string> attachedDimensions)
        {
            _dimension1 = dimension;
            _value = value;
            _attachedDimensions = attachedDimensions;
        }

        public D1 Dimension1 { get { return _dimension1; } }
        public T Value { get { return _value; } }
        public IEnumerable<string> AttachedDimensions { get { return _attachedDimensions; } }
    }

    public struct Parameter2DimensionalMember<D1,D2,T>
    {
        private IEnumerable<string> _attachedDimensions;
        D1 _dimension1;
        D2 _dimension2;
        T _value;

        public Parameter2DimensionalMember(D1 dimension1, D2 dimension2, T value, IEnumerable<string> attachedDimensions)
        {
            _dimension1 = dimension1;
            _dimension2 = dimension2;
            _value=value;
            _attachedDimensions = attachedDimensions;
        }

        public D1 Dimension1 { get { return _dimension1;}}
        
        public D2 Dimension2 { get { return _dimension2;}}

        public T Value { get { return _value;}}

        public IEnumerable<string> AttachedDimensions { get { return _attachedDimensions; } }
    }


    public interface IParameter1Dimensional<D1, T>
    {
        T this[D1 index] { get; }
        IEnumerable<Parameter1DimensionalMember<D1, T>> GetEnumerator();
    }

    public interface IVariable1Dimensional<D1, T>
    {
        T this[D1 index] { get; set; }
        IEnumerable<Parameter1DimensionalMember<D1, T>> GetEnumerator();
    }

    public interface IParameter2Dimensional<D1, D2, T>
    {
        T this[D1 index1, D2 index2] { get; }
        IEnumerable<Parameter2DimensionalMember<D1, D2, T>> GetEnumerator();
    }

    public interface IVariable2Dimensional<D1, D2, T>
    {
        T this[D1 index1, D2 index2] { get; set; }
        IEnumerable<Parameter2DimensionalMember<D1, D2, T>> GetEnumerator();
    }
}
