﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using Esmf;
using System;

namespace Fund
{

    public interface IFundOld2Climate
    {
        Timestep emissionPeriod { get; }
        MarginalGas marginalGas { get; }

        IVariable1Dimensional<Timestep, double> addco2 { get; }
        IVariable1Dimensional<Timestep, double> addch4 { get; }
        IVariable1Dimensional<Timestep, double> addn2o { get; }
        IVariable1Dimensional<Timestep, double> addsf6 { get; }
    }

    public class FundOld2ClimateComponent
    {
        public void Run(Clock clock, IFundOld2Climate state)
        {
            var t = clock.Current;
            var s = state;

            state.addch4[t] = 0;
            state.addco2[t] = 0;
            state.addn2o[t] = 0;
            state.addsf6[t] = 0;

            if ((t.Value >= s.emissionPeriod.Value ) && (t.Value < (s.emissionPeriod.Value + 10)))
            {
                switch (s.marginalGas)
                {
                    case MarginalGas.C: state.addco2[t] = 1; break;
                    case MarginalGas.CH4: state.addch4[t] = 1; break;
                    case MarginalGas.N2O: state.addn2o[t] = 1; break;
                    case MarginalGas.SF6: state.addsf6[t] = 1; break;
                    default: throw new Exception();
                }
            }

        }
    }

}