﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund
{

    // Discounting with a constant pure rate of time preference
    public class ConstantDiscounting : WeightingScheme
    {
        private double m_pureRateOfTimePreference;
        private double m_riskAversion;
        private int m_baseYear;
        private string m_name;

        public override void CalculateWeights(ModelOutput ModelOutput)
        {
            double i_newPerCapitaIncome, i_oldPerCapitaIncome;
            double i_perCapitaGrowthRate;
            double i_discountFactor;

            /*
              {
              i_discountFactor:=1;
              for i:=50 to LegacyConstants.NYear do
              begin
                for l:=1 to LegacyConstants.NoReg do
                  AddWeight(i, l, i_discountFactor);

                i_totalNewIncome:=0;
                i_totalNewPopulation:=0;
                i_totalOldIncome:=0;
                i_totalOldPopulation:=0;
                for l:=1 to LegacyConstants.NoReg do
                begin
                  i_totalNewIncome:=i_totalNewIncome + ModelOutput.Incomes.Item[i, l];
                  i_totalNewPopulation:=i_totalNewPopulation + ModelOutput.Populations.Item[i, l];

                  i_totalOldIncome:=i_totalOldIncome + ModelOutput.Incomes.Item[i-1, l];
                  i_totalOldPopulation:=i_totalOldPopulation + ModelOutput.Populations.Item[i-1, l];
                end;
                i_newPerCapitaIncome:=i_totalNewIncome/i_totalNewPopulation;
                i_oldPerCapitaIncome:=i_totalOldIncome/i_totalOldPopulation;
                i_perCapitaGrowthRate:=(i_newPerCapitaIncome-i_oldPerCapitaIncome)/i_oldPerCapitaIncome;
                i_discountFactor:=i_discountFactor/(1.00 + m_PureRateOfTimePreference + i_perCapitaGrowthRate);
              end;  */

            for (int i = 1; i <= LegacyConstants.NoReg; i++)
            {
                i_discountFactor = 1;
                // TODO -oDavid Anthoff: Change start of "real" period to constant
                for (int l = m_baseYear; l <= LegacyConstants.NYear; l++)
                {
                    AddWeight(l, i, i_discountFactor);
                    i_newPerCapitaIncome = ModelOutput.Incomes[l, i] / ModelOutput.Populations[l, i];
                    i_oldPerCapitaIncome = ModelOutput.Incomes[l - 1, i] / ModelOutput.Populations[l - 1, i];
                    i_perCapitaGrowthRate = (i_newPerCapitaIncome - i_oldPerCapitaIncome) / i_oldPerCapitaIncome;
                    i_discountFactor = i_discountFactor / (1.00 + m_pureRateOfTimePreference + m_riskAversion * i_perCapitaGrowthRate);
                }
            }

        }

        public ConstantDiscounting(double PureRateOfTimePreference)
            : base()
        {
            m_pureRateOfTimePreference = PureRateOfTimePreference;
            m_baseYear = 50;
            m_riskAversion = 1.0;
            m_name = string.Format("constant discount rate [prtp={0}%, risk aversion={1}, base year={2}]", m_pureRateOfTimePreference * 100, m_riskAversion, m_baseYear + 1950);
        }

        public ConstantDiscounting(double PureRateOfTimePreference, int baseYear)
            : base()
        {
            m_pureRateOfTimePreference = PureRateOfTimePreference;
            m_baseYear = baseYear;
            m_riskAversion = 1.0;
            m_name = string.Format("constant discount rate [prtp={0}%, risk aversion={1}, base year={2}]", m_pureRateOfTimePreference * 100, m_riskAversion, m_baseYear + 1950);
        }

        protected override string GetName()
        {
            return m_name;
        }

        public ConstantDiscounting(double PureRateOfTimePreference, int baseYear, string name)
            : base()
        {
            m_pureRateOfTimePreference = PureRateOfTimePreference;
            m_baseYear = baseYear;
            m_name = name;
            m_riskAversion = 1.0;
        }

        public ConstantDiscounting(double PureRateOfTimePreference, double RiskAversion, int baseYear)
        {
            m_pureRateOfTimePreference = PureRateOfTimePreference;
            m_riskAversion = RiskAversion;
            m_baseYear = baseYear;
            m_name = string.Format("constant discount rate [prtp={0}%, risk aversion={1}, base year={2}]", m_pureRateOfTimePreference * 100, m_riskAversion, m_baseYear + 1950);
        }
    }
}