﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Esmf;
using Fund.CommonDimensions;

namespace Fund.Components.ClimateRegional
{
    public interface IClimateRegionalState
    {
        IParameter1Dimensional<Timestep, Double> inputtemp { get; }
        IParameter1Dimensional<Region, Double> bregtmp { get; }
        IParameter1Dimensional<Region, Double> bregstmp { get; }
        IParameter2Dimensional<Timestep, Region, Double> scentemp { get; }

        IVariable2Dimensional<Timestep, Region, Double> temp { get; }
        IVariable2Dimensional<Timestep, Region, Double> regtmp { get; }
        IVariable2Dimensional<Timestep, Region, Double> regstmp { get; }
    }

    public class ClimateRegionalComponent
    {
        public void Run(Clock clock, IClimateRegionalState state)
        {
            var s = state;
            var t = clock.Current;

            foreach (var r in Dimension<Region>.Values)
            {
                s.regtmp[t, r] = s.inputtemp[t] * s.bregtmp[r] + s.scentemp[t, r];
            }

            foreach (var r in Dimension<Region>.Values)
            {
                s.temp[t, r] = s.regtmp[t, r] / s.bregtmp[r];
            }

            foreach (var r in Dimension<Region>.Values)
            {
                s.regstmp[t, r] = s.inputtemp[t] * s.bregstmp[r] + s.scentemp[t, r];
            }
        }

    }
}
