﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Esmf;
using Fund.CommonDimensions;

namespace Fund.Components.Ocean
{

    /// <summary>State for the ocean component</summary>  
    public interface IOceanState
    {
        /// <summary> Sea-level rise in cm </summary>
        IVariable1Dimensional<Timestep, double> sea { get; }

        /// <summary></summary>
        double lifesea { get; }

        /// <summary> </summary>
        double seas { get; }

        /// <summary> Temperature incrase in C° </summary>
        IParameter1Dimensional<Timestep, double> temp { get; }
    }


    public class OceanComponent
    {
        public void Run(Clock clock, IOceanState state)
        {
            var s = state;
            var t = clock.Current;

            // Delay in sea-level rise
            var delaysea = 1.0 / s.lifesea;

            // Calculate sea level rise  
            var ds = delaysea * s.seas * s.temp[t] - delaysea * s.sea[t - 1];

            s.sea[t] = s.sea[t - 1] + ds;

        }
    }
}
