﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Esmf;
using Fund.CommonDimensions;

namespace Fund.Components.ScenarioUncertainty
{

    public interface IScenarioUncertaintyState
    {
        IVariable2Dimensional<Timestep, Region, double> pgrowth { get; }
        IVariable2Dimensional<Timestep, Region, double> ypcgrowth { get; }
        IVariable2Dimensional<Timestep, Region, double> aeei { get; }
        IVariable2Dimensional<Timestep, Region, double> acei { get; }
        IVariable2Dimensional<Timestep, Region, double> ForestEmm { get; }

        Timestep timeofuncertaintystart { get; }

        IParameter2Dimensional<Timestep, Region, double> inputpgrowth { get; }
        IParameter2Dimensional<Timestep, Region, double> inputypcgrowth { get; }
        IParameter2Dimensional<Timestep, Region, double> inputaeei { get; }
        IParameter2Dimensional<Timestep, Region, double> inputacei { get; }
        IParameter2Dimensional<Timestep, Region, double> inputForestEmm { get; }

        IParameter1Dimensional<Region, double> ecgradd { get; }
        IParameter1Dimensional<Region, double> pgadd { get; }
        IParameter1Dimensional<Region, double> aeeiadd { get; }
        IParameter1Dimensional<Region, double> aceiadd { get; }
        IParameter1Dimensional<Region, double> foremadd { get; }
    }


    public class ScenarioUncertaintyComponent
    {
        public void Run(Clock clock, IScenarioUncertaintyState state)
        {
            var s = state;
            var t = clock.Current;

            Double yearsFromUncertaintyStart = t.Value - s.timeofuncertaintystart.Value;
            var sdTimeFactor = (yearsFromUncertaintyStart / 50.0) / (1.0 + (yearsFromUncertaintyStart / 50.0));

            foreach (var r in Dimension<Region>.Values)
            {

                s.ypcgrowth[t, r] = s.inputypcgrowth[t, r] + (t >= s.timeofuncertaintystart ? s.ecgradd[r] * sdTimeFactor : 0.0);
                s.pgrowth[t, r] = s.inputpgrowth[t, r] + (t >= s.timeofuncertaintystart ? s.pgadd[r] * sdTimeFactor : 0.0);
                s.aeei[t, r] = s.inputaeei[t, r] + (t >= s.timeofuncertaintystart ? s.aeeiadd[r] * sdTimeFactor : 0.0);
                s.acei[t, r] = s.inputacei[t, r] + (t >= s.timeofuncertaintystart ? s.aceiadd[r] * sdTimeFactor : 0.0);
                s.ForestEmm[t, r] = s.inputForestEmm[t, r] + (t >= s.timeofuncertaintystart ? s.foremadd[r] * sdTimeFactor : 0.0);
            }

        }
    }
}
